<?php


class My_WC_Cart { //Class Begins

	function __construct(){
		add_action('woocommerce_settings_tabs_array', array($this,'add_tab')); /* Add Tab */
		add_action( 'woocommerce_settings_tabs_quivers_settings', array($this,'load_form') ); /* Load Quivers Form */
		add_action( 'woocommerce_settings_tabs_quivers_tax_settings', array($this,'load_quivers_tax_form') ); /* Load Quivers Tax Form */
		add_action( 'woocommerce_update_options_quivers_settings', array($this, 'update_settings') ); /* Save Settings */
		add_action( 'woocommerce_update_options_quivers_tax_settings', array($this, 'update_tax_settings') ); /* Save Tax Settings */
		add_action( 'woocommerce_cart_calculate_fees', array($this, 'checkout_billing') ); /* Tax Calc */
	
		add_action( 'woocommerce_checkout_create_order_line_item', array($this,'ektgn_meta_to_line_item'), 20, 4 ); 
		add_filter('woocommerce_hidden_order_itemmeta', array($this,'custom_woocommerce_hidden_order_itemmeta'), 10, 1);
	}

	function custom_woocommerce_hidden_order_itemmeta($arr){
		$arr[] = '_item_tax_value';
   		return $arr;
	}

	function ektgn_meta_to_line_item( $item, $cart_item_key, $values, $order )
	{
	    $_p = $item->get_product();
	    $key = '_item_tax_value';
	 	$val = isset($this->qqq['q_tax'][$values['product_id']][$values['variation_id']]) ? json_encode($this->qqq['q_tax'][$values['product_id']][$values['variation_id']]) : 0;
	    if ( false !== ( $value = $_p->get_meta( $key, true ) ) )
	    {
	        $item->add_meta_data( $key , $val , true ); 
	    }
	                 
	}	

	function checkout_billing(){
	    $startTime = microtime(true);
        $log = new My_WC_LogTracking();
		// Call Validate API, if fails call Countries API and calculate taxes.
		if (is_checkout() || is_cart()) {
			//1.Model Data
			$data = $this->model_data(WC()->cart);
            $log->session_start(["Tax Data"=>$data], date('r', $startTime)."UTC");
			$test = 'demo';
			if (get_option('wc_settings_tab_demo_sandbox_status') == 'no') $test = '';

			//2.CallAPI when shipping address
			if (sizeof($data['items']) > 0) {
				$res = $this->CallAPI(
					"POST",
					"https://api.quivers" . $test . ".com/v1/CustomerOrders/Validate",
					json_encode($data),
					[array('Content-Type:application/json')]
				);
				$request =  ["tax_url" => "https://api.quivers" . $test . ".com/v1/CustomerOrders/Validate", "tax_data" => $data];
				$res = json_decode($res['data'], true);
				$endTime = microtime(true);
                $order_detail = ["tax_data"=>$data, "Response"=>$res]   ;
                $log->validate_api_call($request, $res, date('r', $startTime)."UTC", date('r', $endTime)."UTC");
                $log->session_end($order_detail, date('r', $endTime)."UTC");
			} else {
				$res['result']['items'] = [];
			}

			//3.UPDATE $cart_obj
			if (isset($res['result'])) {
				$this->setTaxesFee($res['result']);
				$this->setStatementDescriptor($res['result']['statementDescriptor']);
                $this->setProperty($res['result']['payment']);
			}
			else {
				// if validate API failed, call countries API.
				$response = $this->CallAPI(
					"GET",
					"https://api.quivers" . $test . ".com/v1/Countries",
					false,
					false,
					5
				);
				$request =  ["tax_url" => "https://api.quivers" . $test . ".com/v1/Countries", "tax_data" => $data];
				$response_data = json_decode($response['data'], true);
				if (isset($response_data['result'])) {
					$this->setWorstTaxFee($response_data);
				}
                $endTime = microtime(true);
                $log->countries_api_call($request, $response, date('r', $startTime)."UTC", date('r', $endTime)."UTC");
                $log->session_end(["Tax Data"=>$data, "Response"=>$response["data"]], date('r', $endTime)."UTC");
			}
		}
	}

	function setProperty($payment) {
		$merchantId = '0';
        $destination = false;
        
        if(!empty($payment) && !empty($payment["destination"]))
        {
            $merchantId = $payment["accountId"];
            $destination = $payment["destination"];
        }
        WC()->session->set('destination',$destination);
        WC()->session->set('merchantId',$merchantId);
	}

	function setStatementDescriptor($statement_descriptor){
        // set statement descriptor as a session attribute
        WC()->session->set('statement_descriptor',$statement_descriptor);
    }


	function setTaxesFee($ci) {
		$fees_add = 0;
		$q_tax = [];
		if(isset($ci['items']) && sizeof($ci['items']) > 0){
			foreach ($ci['items'] as $prod) {
				foreach ($prod['pricing']['taxes'] as $k => $value) {
					$variationRefId = $prod['variantRefId'] == '' ? 0 : $prod['variantRefId'];
					$q_tax['q_tax'][$prod['productRefId']][$variationRefId] = isset($value['amount']) && $value['amount'] > 0 ? $value['amount'] : 0;
					$fees_add += isset($value['amount']) && $value['amount'] > 0 ? $value['amount'] : 0;
				}
			}
			$this->qqq = $q_tax;
			WC()->cart->add_fee( __('Tax', 'woocommerce'), $fees_add, false );
		} else{
			
		}
	}

	function setWorstTaxFee($countries_response_data) {
		if (empty($countries_response_data)) {
			return;
		}

		$cart = WC()->cart;
		$order_country_code = WC()->customer->get_shipping_country();
		$order_region_code = WC()->customer->get_shipping_state();
		$products = $cart->get_cart();

		// get worst tax rate.
		$country_max_tax_rate = NULL;
		foreach ($countries_response_data['result'] as $value) {
		  if ($value['abbreviations']['two'] === $order_country_code) {
		    foreach ($value['regions'] as $region_value) {
		      if (strcasecmp($region_value['abbreviation'], $order_region_code) == 0) {
		        $country_max_tax_rate = (float) $region_value['maxTaxRate'];
		        break;
		      }
		    }
		    // Quivers supports "N/A" regions for some countries.
		    if (!$country_max_tax_rate && array_key_exists('0', $value['regions'])) {
		      $country_max_tax_rate = (float) $value['regions']["0"]["maxTaxRate"];
		    }
		  }
		  if ($country_max_tax_rate) {
		    break;
		  }
		}
		if ($country_max_tax_rate === NULL) {
			return;
		}

		// calculate and apply worst tax rate.
		$fees_add = 0;
		$qqq_data = [];
		foreach($products as $product => $values){
			$item_id = $values['variation_id'] == 0 ? $values['product_id'] : $values['variation_id'];
			$item_price = (float)get_post_meta($item_id , '_sale_price', true) ? (float)get_post_meta($item_id , '_sale_price', true) : (float)get_post_meta($item_id , '_price', true);
			$item_quantity = (int)$values['quantity'];

			// creating q_tax data.
			$variationRefId = $values['variation_id'] == '' ? 0 : $values['variation_id'];
			$tax_amount = $item_price * $country_max_tax_rate * $item_quantity;
			$qqq_data['q_tax'][$values['product_id']][$variationRefId] = $tax_amount;

			// order total tax.
			$fees_add += $tax_amount;
		}
		$this->qqq = $qqq_data;
		$cart->add_fee( __('Tax', 'woocommerce'), $fees_add, false );

	}

	function add_tab($settings_tabs){
		$settings_tabs['quivers_settings'] = __( 'Quivers', 'woocommerce-quivers_settings' );
		$settings_tabs['quivers_tax_settings'] = __( 'Quivers Tax', 'woocommerce-quivers_tax_settings' );
		do_action('quivers_add_tab');
		return $settings_tabs;
	}

	function load_form() {
		woocommerce_admin_fields( $this->get_settings() );
	}

	function load_quivers_tax_form() {
		woocommerce_admin_fields( $this->get_quivers_tax_settings() );
	}

	function get_settings() {
		$settings = array(
	        'quivers_settings' => array(
	            'name'     => __( 'Quivers Integration Settings', 'woocommerce-quivers_settings' ),
	            'type'     => 'title',
	            'desc'     => '',
	            'id'       => 'wc_settings_tab_demo_section_title'
	        ),
	        'claiming_groups' => array(
	            'name' => __( 'Claiming Groups:', 'woocommerce-quivers_settings' ),
	            'type' => 'text',
	            'desc_tip' => __( 'Please enter comma seperated values for more than one Claiming Group', 'woocommerce' ),
	            'id'   => 'wc_settings_tab_demo_claiming_groups',
	        ),
	        'apikey' => array(
	            'name' => __( 'Quivers API key:', 'woocommerce-quivers_settings' ),
	            'type' => 'text',
	            'id'   => 'wc_settings_tab_demo_apikey'
	        ),
	        'business_refid' => array(
	            'name' => __( 'Business RefId:', 'woocommerce-quivers_settings' ),
	            'type' => 'text',
	            'id'   => 'wc_settings_tab_demo_business_refid'
	        ),
	        'wp_base_url' => array(
	            'name' => __( 'WP Integration Base URL:', 'woocommerce-quivers_settings' ),
	            'type' => 'text',
	            'id'   => 'wc_settings_tab_demo_wp_base_url'
	        ),
			'wp_username' => array(
	            'name' => __( 'WP Integration Username:', 'woocommerce-quivers_settings' ),
	            'type' => 'text',
	            'id'   => 'wc_settings_tab_demo_wp_username'
	        ),
	        'wp_auth_token' => array(
	            'name' => __( 'WP Integration Auth Token:', 'woocommerce-quivers_settings' ),
	            'type' => 'text',
	            'id'   => 'wc_settings_tab_demo_wp_auth_token'
	        ),
	        'sandbox_status' => array(
	            'name' => __( 'Quivers Sandbox Enabled:', 'woocommerce-quivers_settings' ),
	            'type' => 'select',
	            'id'   => 'wc_settings_tab_demo_sandbox_status',
	            'options'       => array(
			    	'yes'		=> __( 'Yes', 'wps' ),
			        'no'	=> __( 'No', 'wps' )
			    )
	        ),
	        'section_end' => array(
	             'type' => 'sectionend',
	             'id' => 'wc_settings_tab_demo_section_end'
	        )
	    );
		return apply_filters( 'wc_settings_tab_demo_settings', $settings );
	}

	function get_quivers_tax_settings() {
		$quivers_marketplaces = get_option('wc_quivers_marketplaces');

		$tax_settings = array(
	        'quivers_tax_settings' => array(
	            'name' => __( 'Quivers Tax Settings', 'woocommerce-quivers_tax_settings' ),
	            'type' => 'title',
	            'desc' => '',
	            'id'   => 'wc_settings_tab_tax_section_title'
	        ),
	        'usd' => array('name' => __( 'USD Marketplace:', 'woocommerce-quivers_tax_settings' ),
				'type' => 'select',
				'id'   => 'wc_settings_tab_tax_quivers_marketplace_usd',
				'options' => $quivers_marketplaces
	        ),
	        'eur' => array(
				'name'    => __( 'EUR Marketplace:', 'woocommerce-quivers_tax_settings' ),
				'type'    => 'select',
				'id'      => 'wc_settings_tab_tax_quivers_marketplace_eur',
				'options' => $quivers_marketplaces
	        ),
			'aud' => array(
				'name'    => __( 'AUD Marketplace:', 'woocommerce-quivers_tax_settings' ),
				'type'    => 'select',
				'id'      => 'wc_settings_tab_tax_quivers_marketplace_aud',
				'options' => $quivers_marketplaces
	        ),
			'jpy' => array(
				'name'    => __( 'JPY Marketplace:', 'woocommerce-quivers_tax_settings' ),
				'type'    => 'select',
				'id'      => 'wc_settings_tab_tax_quivers_marketplace_jpy',
				'options' => $quivers_marketplaces
	        ),
			'gbp' => array(
				'name'    => __( 'GBP Marketplace:', 'woocommerce-quivers_tax_settings' ),
				'type'    => 'select',
				'id'      => 'wc_settings_tab_tax_quivers_marketplace_gbp',
				'options' => $quivers_marketplaces
	        ),
	        'cad' => array(
				'name'    => __( 'CAD Marketplace:', 'woocommerce-quivers_tax_settings' ),
				'type'    => 'select',
				'id'      => 'wc_settings_tab_tax_quivers_marketplace_cad',
				'options' => $quivers_marketplaces
	        ),
	        'section_end' => array(
				'type' => 'sectionend',
				'id'   => 'wc_settings_tab_tax_section_end'
	        )
	    );
		
		return apply_filters( 'wc_settings_tab_tax_settings', $tax_settings );
	}

	function profileCreate($result){
		$request_data = [
	      "client_type" => "Woocommerce",
	      "base_url" => $result['wp_base_url'],
	      "business_refid" => $result['business_refid'],
	      "api_key" => $result['apikey'],
	      "consumer_key" => $result['wp_username'],
	      "consumer_secret" => $result['wp_auth_token']
	    ];

		$test = 'demo';
		if(get_option('wc_settings_tab_demo_sandbox_status') == 'no') $test = '';
		
		$response = $this->CallAPI(
			"POST",
			"https://middleware.quivers" . $test . ".com/profile/create",
			json_encode($request_data),
			[array('Content-Type:application/json')]
		);
		
		if($response['httpcode'] != 200){
			echo "<div class='error notice'><p><strong>Unable to sync to Quivers.</strong></p></div>";
			return NULL;
		}

		$response_data = json_decode($response['data'], true);
		return $response_data['uuid'];

	}

	function profileUpdate() {
		$currencies = ["usd", "eur", "aud", "jpy", "gbp", "cad"];
		$claiming_groups = explode(",", get_option('wc_settings_tab_demo_claiming_groups'));
		$marketplaces_request_data = [];

		foreach ($currencies as $currency) {
			if (!get_option('wc_settings_tab_tax_quivers_marketplace_' . $currency)) {
				continue;
			}

			$data = [];
			$data['currency'] = $currency;
			$data['marketplace_id'] = get_option('wc_settings_tab_tax_quivers_marketplace_' . $currency);
			$data['claiming_group_ids'] = $claiming_groups;
			array_push($marketplaces_request_data, $data);
		}

		$request_data = [
			"marketplaces" => $marketplaces_request_data,
		];
		$headers = [
			array('Content-Type:application/json'),
			array('uuid:' . get_option('wc_middleware_profile_id')),
		];

		$test = 'demo';
		if (get_option('wc_settings_tab_demo_sandbox_status') == 'no') $test = '';

		$response = $this->CallAPI(
			"POST",
			"https://middleware.quivers" . $test . ".com/profile/update/marketplaces",
			json_encode($request_data),
			$headers
		);

		if($response['httpcode'] != 200) {
			echo "<div class='error notice'><p><strong>Unable to sync Quivers Profile. Please verify that your are connected to Quivers.</strong></p></div>";
		}
	}

	function update_settings() {
		woocommerce_update_options( $this->get_settings() );

		$result['apikey'] = get_option('wc_settings_tab_demo_apikey');
		$result['claiming_groups'] = get_option('wc_settings_tab_demo_claiming_groups');
		$result['business_refid'] = get_option('wc_settings_tab_demo_business_refid');
		$result['wp_auth_token'] = get_option('wc_settings_tab_demo_wp_auth_token');
		$result['wp_username'] = get_option('wc_settings_tab_demo_wp_username');
		$result['wp_base_url'] = get_option('wc_settings_tab_demo_wp_base_url');
		$result['sandbox_status'] = get_option('wc_settings_tab_demo_sandbox_status');
		
		// save Quivers Marketplaces in db
		$quivers_marketplaces = $this->getQuiversMarketplaces($result);
		if ( !$quivers_marketplaces ) {
			return;
		}
		$this->getOrCreateOption('wc_quivers_marketplaces', $quivers_marketplaces);

		$middleware_profile_id = $this->profileCreate($result);
		$this->getOrCreateOption('wc_middleware_profile_id', $middleware_profile_id);

	}

	function update_tax_settings($result) {
		woocommerce_update_options( $this->get_quivers_tax_settings() );
		$this->profileUpdate();

	}

	function getQuiversMarketplaces($result) {
		$quivers_marketplaces = [
			'' => '- Select a Marketplace -'
		];

		if ( $result['sandbox_status'] == 'no' ) {
			$url = "https://cloudhub-internal.quivers.com/api/v1/private/ProductGroups/MyProductGroups?refId=" . $result['business_refid'];
		}
		else {
			$url = "https://cloudhub.quiversdemo.com/api/v1/private/ProductGroups/MyProductGroups?refId=" . $result['business_refid'];
		}

		$response = $this->CallAPI(
			"GET",
			$url,
			false,
			[array('Authorization: apikey ' . $result['apikey'])]
		);

		if ( $response['httpcode'] != 200 ) {
			echo "<div class='error notice'><p><strong>Unable to fetch Quivers Marketplaces. Please verify the configuration values.</strong></p></div>";
			return $quivers_marketplaces;
		}

		$product_groups = json_decode($response['data'], true);

		foreach ($product_groups as $product_group) {
		  if ($product_group['Type'] === 'Marketplace') {
		    $quivers_marketplaces = array_merge($quivers_marketplaces, [$product_group['Id'] => $product_group['Name']]);
		  }
		}

		return $quivers_marketplaces;
	}

	function model_data( $input ){
		$data = [];

		set_error_handler(array($this,'exceptions_error_handler'));
		try{
			$marketplace_id = get_option('wc_settings_tab_tax_quivers_marketplace_' . get_woocommerce_currency());
			if (!$marketplace_id) {
				return $data;
			}
			$data['marketplaceId'] = $marketplace_id;
			$data['customer']['firstname'] = WC()->customer->get_shipping_first_name();
			$data['customer']['lastname'] = WC()->customer->get_shipping_last_name();
			$data['customer']['email'] = WC()->customer->get_billing_email();
			$data['shippingAddress']['firstname'] = WC()->customer->get_shipping_first_name();
			$data['shippingAddress']['lastname'] = WC()->customer->get_shipping_last_name();
			$data['shippingAddress']['email'] = WC()->customer->get_billing_email();
			$data['shippingAddress']['company'] = WC()->customer->get_shipping_company();
			$data['shippingAddress']['phoneNumber'] = WC()->customer->get_billing_phone();
			$data['shippingAddress']['line1'] = WC()->customer->get_shipping_address_1();
			$data['shippingAddress']['line2'] = WC()->customer->get_shipping_address_2();
			$data['shippingAddress']['city'] = WC()->customer->get_shipping_city();
			$data['shippingAddress']['postCode'] = WC()->customer->get_shipping_postcode();
			$data['shippingAddress']['country'] = WC()->customer->get_shipping_country();
			$data['shippingAddress']['region'] = WC()->customer->get_shipping_state();

			$prods = $input->get_cart();
			$amount = strip_tags(WC()->cart->get_cart_shipping_total());
			$shipping_amt_total = (float)substr($amount, 5, strlen($amount));
			$shipping_amt_per_item = (float) $shipping_amt_total/sizeof($prods);
			$shipping_amt_per_item = floatval(substr($shipping_amt_per_item, 0, strpos($shipping_amt_per_item, '.') + 3));
			$shipping_amt_diff = round($shipping_amt_total - ($shipping_amt_per_item * sizeof($prods)), 2);

			foreach($prods as $prod => $values){

				$_product =  wc_get_product( $values['data']->get_id() );
		
				$prod_var_opt = array();
				if(sizeof($values['variation']) > 0){
					$i=0;
					foreach ($values['variation'] as $key => $value) {
						$prod_var_opt[$i]["name"] = $key;
						$prod_var_opt[$i]["value"]["name"] = $value;
						$i++;
					}
				}

				$shipping_fees = array();

				$shipping_fees["name"] = "FedEx 2-Day";
				$shipping_fees["amount"] = $shipping_amt_per_item;

				$items = array();
				$items["product"]["name"] = $_product->get_title();
				$items["product"]["refId"] = $values['product_id'];
				$items["product"]["variant"]["name"] = $_product->get_name();
				$items["product"]["variant"]["refId"] = $values['variation_id'];
				$items["product"]["variant"]["option"][] = $prod_var_opt;
				$items["quantity"] = (float)$values['quantity'];
				$type_id = $values['variation_id'] == 0 ? $values['product_id'] : $values['variation_id'];
				$items["pricing"]["unitPrice"] = (float)get_post_meta($type_id , '_price', true);
				$items["pricing"]["salePrice"] = (float)get_post_meta($type_id , '_sale_price', true);
				$items["pricing"]["shippingFees"][] = $shipping_fees;
				$items["pricing"]["customFees"] = [];

				if ( sizeof(WC()->cart->get_applied_coupons()) > 0 ) {
					foreach (WC()->cart->cart_contents as $key => $item_data) {
						if ($item_data['product_id'] != $values['product_id']) {
							continue;
						}

						$pricing_disc = [];
						$pricing_disc[0]["name"] = 'Discount';
						$pricing_disc[0]["amount"] = -($item_data['line_subtotal'] - $item_data['line_total']);
						$items["pricing"]["discounts"] = $pricing_disc;
						break;
					}
				}


				$data["items"][] = $items;
			}

			foreach ($data["items"] as &$payload_item) {
				if (!$shipping_amt_diff) {
					break;
				}

				$payload_item["pricing"]["shippingFees"][0]["amount"] += 0.01;
				$shipping_amt_diff -= 0.01;
			}
			unset($payload_item);

		} catch(Exception $e){
			restore_error_handler();
		}
		restore_error_handler();

		return $data;
	}
	////////////////////////////////////////////////////////////////////////////
	/////////////////////////Important Functions////////////////////////////////
	////////////////////////////////////////////////////////////////////////////
	function CallAPI($method, $url, $data = false, $headers = false, $timeout = 10)
	{
		$curl = curl_init();

		switch ($method)
		{
			case "POST":
				curl_setopt($curl, CURLOPT_POST, 1);
				// curl_setopt( $curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
				if ($data)
					curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
				break;
			case "PUT":
				curl_setopt($curl, CURLOPT_PUT, 1);
				break;
			default:
				if ($data)
					$url = sprintf("%s?%s", $url, http_build_query($data));
		}

		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $timeout - 1);
		curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
		if ($headers) {
			foreach ($headers as $header) {
				curl_setopt( $curl, CURLOPT_HTTPHEADER, $header);
			}
		}

		$res = [];
		try{
			$result = curl_exec($curl);
		}
		catch(Exception $e) {
			return $res;
		}

		$httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		$res['data'] = $result;
		$res['httpcode'] = @$httpcode;

		return $res;
	}

	function getOrCreateOption($option, $option_value) {
		if (get_option($option)) {
			update_option($option, $option_value);
	    }
	    else {
			$is_added = add_option($option, $option_value, '', 'yes');
			if (!$is_added) {
				// returns false if an option with the same name exists
				// try deleting and adding again
				delete_option($option);
				add_option($option, $option_value, '', 'yes');
			}
	    }
	}

	function add_menu_items() {
		add_menu_page('Settings','Quivers','manage_options','quivers_taxes',array($this,'load_form'));
	}

	function exceptions_error_handler($severity, $message, $filename, $lineno) {
		if ($severity) {
			throw new ErrorException($message, 0, $severity, $filename, $lineno);
		}
	}
	/*--------------------------------ENDS---------------------------------*/					

}//Class Ends

// finally instantiate our plugin class and add it to the set of globals
if( class_exists("My_WC_Cart") ) $GLOBALS['my_wc_cart'] = new My_WC_Cart();

?>